#!/usr/bin/env python3

##########################################
######## JUST STARTED/ IGNORE THIS FILE!!!
##########################################


# Unit Tests for machinelocalwatchdog.py

import sys
import os
import errno
import tempfile
import unittest
import hashlib

from machinelocalwatchdog import MachineLocalWatchdog

# if script's parent directory isn't in sys.path, then add it
if ('/'.join(sys.path[0].split("/")[:-1]) not in sys.path):
    sys.path.append('/'.join(sys.path[0].split("/")[:-1]))


class MachineLocalWatchdogTests(unittest.TestCase):

    # Create temporary files, test data and populate temp files with test data
    def setUp(self):
        self.csvsuffix = ".csv"

        self.tmpcsvdir_test = self.__create_tmp_dir()
        self.tmpcsvdir_test2 = self.__create_tmp_dir()
        self.tmpcsvfile_test = self.__create_tmp_file(self.tmpcsvdir_test.name, "", self.csvsuffix)
        self.tmpcsvfile_test2 = self.__create_tmp_file(self.tmpcsvdir_test2.name, "", self.csvsuffix)

        self.csv_data = "MachineName,Ok,Field,Details and stuff\n"
        self.csv_data += "MachineName,Warning,Field2,More Details\n"

        self.assertTrue(self.__populate_csv_file(self.tmpcsvfile_test2.name, self.csv_data))

    # Delete temp files and directories
    def tearDown(self):
        self.__silent_delete_file(self.tmpcsvfile_test)
        self.__silent_delete_file(self.tmpcsvfile_test2)
        self.__silent_delete_dir(self.tmpcsvdir_test)
        self.__silent_delete_dir(self.tmpcsvdir_test2)

    # Create a tmp dir
    @staticmethod
    def __create_tmp_dir() -> tempfile.TemporaryDirectory():
        tmpdir = tempfile.TemporaryDirectory()
        assert isinstance(tmpdir, object)
        print("\nTemporary Directory: " + tmpdir.name)

        return tmpdir

    # Create a tmp file
    @staticmethod
    def __create_tmp_file(tmpdir: tempfile.TemporaryDirectory,
                          fileprefix: str,
                          filesuffix: str) -> tempfile.NamedTemporaryFile:

        tmpfile = tempfile.NamedTemporaryFile(suffix=filesuffix, prefix=fileprefix, dir=tmpdir, delete=False)
        assert isinstance(tmpfile, object)

        print("\nTemporary Filename: " + tmpfile.name)

        return tmpfile

    # Populate tmp csv file with prepared data
    @staticmethod
    def __populate_csv_file(file: tempfile.NamedTemporaryFile, csv_data: str) -> bool:

        print("\nTest Configuration Data:")
        print(csv_data)

        try:
            with open(file, 'w', newline="\r\n") as outfile:
                outfile.write(csv_data)
        except Exception as e:
            print("test_machinelocalwatchdog.__populate_csv_file: Exception: " + str(e))
            return False

        return True

    # Delete a tmp file
    @staticmethod
    def __silent_delete_file(tmpfile: tempfile.NamedTemporaryFile):
        print("\nDelete: " + tmpfile.name)
        tmpfile.close()

        try:
            os.remove(tmpfile.name)
        except OSError as e:
            if e.errno != errno.ENOENT:  # errno.ENOENT = no such file or directory
                raise  # re-raise exception if a different error occured

    # Delete a tmp dir
    @staticmethod
    def __silent_delete_dir(tmpdir: tempfile.TemporaryDirectory):
        print("\nDelete: " + tmpdir.name)
        tmpdir.cleanup()

    # Compare all file checksums
    def __compare_file_sha256_checksums(self, files_to_compare):
        results = []

        # Loop through all files being compared and place their checksum into the results[] list
        for idx, file in enumerate(files_to_compare):
            try:
                with open(file, "rb") as infile:
                    file_data = infile.read()
                    print("\nFile" + str(idx) + " Data:")
                    print(file_data)
            except Exception as e:
                print("test_machinelocalwatchdog.__compare_file_sha256_checksums: Exception: " + str(e))
                return False

            hash = hashlib.new("sha256")
            hash.update(file_data)
            file_hash = hash.hexdigest()
            print("\nFile" + str(idx) + " Hash: " + str(file_hash))

            results.append(file_hash)

        # Convert to a set which elliminates all duplicate values in a list.  If all values in the list match
        # then the set will contain exactly one element, which is what we're looking for.
        # If more than one element exists that means more than one checksum was found and the files are not the same
        if len(set(results)) != 1:
            return False

        return True

    def test_copy_initial_boot_file(self):
        print("test_copy_initial_boot_file")
        self.machinelocalwatchdog = MachineLocalWatchdog(self.tmpcsvfile_test2.name, self.tmpcsvfile_test.name,
                                                         self.tmpcsvdir_test2.name,
                                                         self.tmpcsvfile_test2.name, "", True)

        result = self.machinelocalwatchdog.copy_initial_boot_file()

        self.assertTrue(result)

        files_to_compare = [self.tmpcsvfile_test.name, self.tmpcsvfile_test2.name]
        self.assertTrue(self.__compare_file_sha256_checksums(files_to_compare))

    def test_update_watchdogstatus_file(self):
        print("test_update_watchdogstatus_file")
        self.machinelocalwatchdog = MachineLocalWatchdog(self.tmpcsvdir_test.name, self.tmpcsvfile_test.name,
                                                         self.tmpcsvdir_test2.name,
                                                         self.tmpcsvfile_test.name, "", True)

        result = self.machinelocalwatchdog.update_watchdogstatus_file()

        self.assertTrue(result)

        self.assertTrue(self.__compare_file_sha256_checksums(self.tmpcsvfile_test.name, self.tmpcsvfile_test2.name))


if __name__ == '__main__':
    unittest.main()

